using System;
using System.Collections;
using System.IO;
using System.Threading;
using EMISecurity.Utilities;

namespace EMISecurity.Security
{
	///<summary>A file/folder shredder library that supports the following algorithm: Simple, British_HMG_IS5_Base_Line, Russian_GOST_P50739, US_Air_Force_5020, British_HMG_IS5_Enhanced, US_DOD_5220_22_M, US_Army_AR380_19, German_VSITR, Canadian_RCMP_TSSIT_OPS_II, USDOD_5220_22_M_E_C_E, Bruce_Schneier, Peter_Gutmann_Lite, Peter_Gutmann.
	///</summary>
	public class Shredder : IDisposable
	{
		internal const int fileBlock = 8192;

		///<summary>Callback to return progress of shred operation.
		///</summary>
		///<param name="pathName"></param>
		public delegate void CallbackProgress(string pathName);

		///<summary>File/folder shredder method
		///</summary>
		public enum ShredMethod
		{
			///<summary>Fast (1 pass)
			///</summary>
			/// <remarks>The fastest wiping scheme. Your data is overwritten with zeroes.</remarks>
			Simple,
			///<summary>British HMG IS5 (Baseline) (1 pass)
			///</summary>
			/// <remarks>Your data is overwritten with zeroes with verification.</remarks>
			British_HMG_IS5_Base_Line,

			///<summary>Russian GOST P50739-95 (2 passes)
			///</summary>
			/// <remarks>GOST P50739-95 wiping scheme calls for a single pass of zeroes followed by a single pass of random byte.</remarks>
			Russian_GOST_P50739,
			/// <summary>
			/// US Air Force 5020 (3 passes)
			/// </summary>
			/// <remarks>Three overwrites based upon user specified character.</remarks>
			US_Air_Force_5020,
			///<summary>British HMG IS5 (Enhanced) (3 passes)
			///</summary>
			/// <remarks>British HMG IS5 (Enhanced) is a three pass overwriting algorithm: first pass - with zeroes, second pass - with ones and the last pass with random bytes (last pass is verified).</remarks>
			British_HMG_IS5_Enhanced,
			///<summary>US Department of Defense DoD 5220.22-M (3 passes)
			///</summary>
			/// <remarks>DoD 5220.22-M is three pass overwriting algorithm: first pass - with zeroes, second pass - with ones and the last pass with random bytes. With all passes verification.</remarks>
			US_DOD_5220_22_M,
			///<summary>US Army AR380-19 (3 passes)
			///</summary>
			/// <remarks>AR380-19 is data wiping scheme specified and published by the U.S. Army. AR380-19 is three pass overwriting algorithm: first pass - with random bytes, second and third passes with certain bytes and with its compliment (with last pass verification) .</remarks>
			US_Army_AR380_19,

			/// <summary>
			/// German VSITR (7 passes)
			/// </summary>
			/// <remarks>The German standard calls for each sector to be overwritten with three alternating patterns of zeroes and ones and in the last pass with character.</remarks>
			German_VSITR,
			/// <summary>
			/// Canadian RCMP TSSIT OPS-II (7 passes)
			/// </summary>
			/// <remarks>RCMP TSSIT OPS-II is seven pass overwriting algorithm with three alternating patterns of zeroes and ones and the last pass - with random character (with last pass verification).</remarks>
			Canadian_RCMP_TSSIT_OPS_II,
			///<summary>the US Department of Defense DoD 5220.22-M(ECE) (7 passes)
			///</summary>
			/// <remarks>DoD 5220.22-M(ECE) is seven pass overwriting algorithm: first and second passes - with certain bytes and with its compliment, then two passes with random character, then two passes with character and its complement and the last pass - with random character.</remarks>
			USDOD_5220_22_M_E_C_E,
			///<summary>Bruce Schneier (7 passes)
			///</summary>
			/// <remarks>The Bruce Schneier wiping algorithm has seven passes: first pass - with ones, the second pass - with zeroes and then five times with random characters.</remarks>
			Bruce_Schneier,
			/// <summary>
			/// Peter Gutmann Lite (10 passes)
			/// </summary>
			/// <remarks>Peter Gutmann Lite wiping algorithm has 10 passes.</remarks>
			Peter_Gutmann_Lite,
			/// <summary>
			/// Peter Gutmann (35 passes)
			/// </summary>
			/// <remarks>Peter Gutmann wiping algorithm has 35 passes.</remarks>
			Peter_Gutmann,
		}

		private Hashtable catchBytes = new Hashtable();

		///<summary>Shred file routine
		///</summary>
		///<param name="fileInfo">FileInfo of the file to be shredded</param>
		///<param name="shredMethod">Shred method</param>
		///<param name="callbackProgress">Callback progress; use null if not needed</param>
		///<returns>true if the file is successfully shredded.</returns>
		///<exception cref="IOException"></exception>
		public bool ShredFile(FileInfo fileInfo, ShredMethod shredMethod, CallbackProgress callbackProgress)
		{
			try
			{
				if ((fileInfo.Attributes & FileAttributes.ReadOnly) != 0)
				{
					FileAttributes fileAttributes = fileInfo.Attributes - 1;
					File.SetAttributes(fileInfo.FullName, fileAttributes);
				}
				switch (shredMethod)
				{
					case ShredMethod.Simple:
						Simple(fileInfo, callbackProgress);
						break;
					case ShredMethod.British_HMG_IS5_Base_Line:
						British_HMG_IS5_Base_Line(fileInfo, callbackProgress);
						break;
					case ShredMethod.Russian_GOST_P50739:
						Russian_GOST_P50739(fileInfo, callbackProgress);
						break;
					case ShredMethod.US_Air_Force_5020:
						US_Air_Force_5020(fileInfo, callbackProgress);
						break;
					case ShredMethod.British_HMG_IS5_Enhanced:
						British_HMG_IS5_Enhanced(fileInfo, callbackProgress);
						break;
					case ShredMethod.US_DOD_5220_22_M:
						US_DOD_5220_22_M(fileInfo, callbackProgress);
						break;
					case ShredMethod.US_Army_AR380_19:
						US_Army_AR380_19(fileInfo, callbackProgress);
						break;
					case ShredMethod.German_VSITR:
						German_VSITR(fileInfo, callbackProgress);
						break;
					case ShredMethod.Canadian_RCMP_TSSIT_OPS_II:
						Canadian_RCMP_TSSIT_OPS_II(fileInfo, callbackProgress);
						break;
					case ShredMethod.USDOD_5220_22_M_E_C_E:
						USDOD_5220_22_M_E_C_E(fileInfo, callbackProgress);
						break;
					case ShredMethod.Bruce_Schneier:
						Bruce_Schneier(fileInfo, callbackProgress);
						break;
					case ShredMethod.Peter_Gutmann_Lite:
						Peter_Gutmann_Lite(fileInfo, callbackProgress);
						break;
					case ShredMethod.Peter_Gutmann:
						Peter_Gutmann(fileInfo, callbackProgress);
						break;
				}
				File.Delete(fileInfo.FullName);
			}
			catch (Exception e)
			{
				throw new IOException(string.Format("Error occurred when shredding {0}.", fileInfo.FullName),
				                      e.GetBaseException());
			}
			return true;
		}

		///<summary>Shred file routine
		///</summary>
		///<param name="fullFillName">File name in full path.</param>
		///<param name="shredMethod">Shredder method</param>
		///<param name="callbackProgress">Callback progress; use null if not needed</param>
		///<returns>true if the file is successfully shredded.</returns>
		///<exception cref="FileNotFoundException"></exception>
		public bool ShredFile(string fullFillName, ShredMethod shredMethod, CallbackProgress callbackProgress)
		{
			if (!File.Exists(fullFillName))
				throw new FileNotFoundException(string.Format("File {0} doesn't exist.", fullFillName));
			FileInfo fi = new FileInfo(fullFillName);
			return ShredFile(fi, shredMethod, callbackProgress);
		}

		private void Simple(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			ShredFileByByte(fileInfo, callbackProgress, 0xff);
		}

		private void British_HMG_IS5_Base_Line(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			ShredFileByByte(fileInfo, callbackProgress, 0x0);
		}

		private static void Russian_GOST_P50739(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			ShredFileByRandomByte(fileInfo, callbackProgress);
		}

		private byte[] CacheBytes(byte byte2write)
		{
			string key = "key" + byte2write;
			byte[] bytes = new byte[fileBlock];

			if (!catchBytes.ContainsKey(key))
			{
				for (int i = 0; i < fileBlock; i++)
				{
					bytes[i] = byte2write;
				}
				catchBytes.Add(key, bytes);
				return bytes;
			}
			else
			{
				return (byte[]) catchBytes[key];
			}
		}

		private byte[] CacheBytesForGutmann(byte byte2write1, byte byte2write2, byte byte2write3)
		{
			string key = "key" + byte2write1 + "_" + byte2write2 + "_" + byte2write3;
			byte[] bytes = new byte[fileBlock];
			int SelectByte = 1;
			if (!catchBytes.ContainsKey(key))
			{
				for (int i = 0; i < fileBlock; i++)
				{
					switch (SelectByte)
					{
						case 1:
							bytes[i] = byte2write1;
							SelectByte = 2;
							break;
						case 2:
							bytes[i] = byte2write2;
							SelectByte = 3;
							break;
						case 3:
							bytes[i] = byte2write3;
							SelectByte = 1;
							break;
					}
				}
				catchBytes.Add(key, bytes);
				return bytes;
			}
			else
			{
				return (byte[]) catchBytes[key];
			}
		}

		private void ShredFileByGutmannByte(FileInfo fileInfo, CallbackProgress callbackProgress, byte byte2write1,
		                                    byte byte2write2, byte byte2write3)
		{
			long fileLength = fileInfo.Length;
			int lastBlock = (int) (fileLength%fileBlock);
			byte[] bytes = new byte[fileBlock];
			byte[] lastBlockBytes = new byte[lastBlock];
			long modWrite = (int) (fileLength/fileBlock);
			CacheBytesForGutmann(byte2write1, byte2write2, byte2write3).CopyTo(bytes, 0);
			//for (int i = 0; i < fileBlock; i++)
			//{
			//    bytes[i] = byte2write;
			//}
			int SelectByte = 1;
			for (int i = 0; i < lastBlock; i++)
			{
				switch (SelectByte)
				{
					case 1:
						lastBlockBytes[i] = byte2write1;
						SelectByte = 2;
						break;
					case 2:
						lastBlockBytes[i] = byte2write2;
						SelectByte = 3;
						break;
					case 3:
						lastBlockBytes[i] = byte2write3;
						SelectByte = 1;
						break;
				}
			}
			if (callbackProgress != null)
				callbackProgress(fileInfo.FullName);
			Stream st = File.OpenWrite(fileInfo.FullName);
			for (long i = 0; i < modWrite; i++)
			{
				st.Write(bytes, 0, fileBlock);
				st.Flush();
			}
			st.Write(lastBlockBytes, 0, lastBlock);
			st.Close();
			st.Dispose();
			//FlushBuffer(fileInfo.FullName, bytes);
		}

		private void ShredFileByByte(FileInfo fileInfo, CallbackProgress callbackProgress, byte byte2write)
		{
			long fileLength = fileInfo.Length;
			int lastBlock = (int) (fileLength%fileBlock);
			byte[] bytes = new byte[fileBlock];
			byte[] lastBlockBytes = new byte[lastBlock];
			long modWrite = (int) (fileLength/fileBlock);
			CacheBytes(byte2write).CopyTo(bytes, 0);
			//for (int i = 0; i < fileBlock; i++)
			//{
			//    bytes[i] = byte2write;
			//}
			for (int i = 0; i < lastBlock; i++)
			{
				lastBlockBytes[i] = byte2write;
			}
			if (callbackProgress != null)
				callbackProgress(fileInfo.FullName);
			Stream st = File.OpenWrite(fileInfo.FullName);
			for (long i = 0; i < modWrite; i++)
			{
				st.Write(bytes, 0, fileBlock);
				st.Flush();
			}
			st.Write(lastBlockBytes, 0, lastBlock);
			st.Close();
			st.Dispose();
			//FlushBuffer(fileInfo.FullName, bytes);
		}

		private static void ShredFileByRandomByte(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			long fileLength = fileInfo.Length;
			int lastBlock = (int) (fileLength%fileBlock);
			byte[] bytes = new byte[fileBlock];
			byte[] lastBlockBytes = new byte[lastBlock];
			long modWrite = (int) (fileLength/fileBlock);
			for (int i = 0; i < fileBlock; i++)
			{
				bytes[i] = RandomByte();
			}
			for (int i = 0; i < lastBlock; i++)
			{
				lastBlockBytes[i] = RandomByte();
			}
			if (callbackProgress != null)
				callbackProgress(fileInfo.FullName);
			Stream st = File.OpenWrite(fileInfo.FullName);
			for (long i = 0; i < modWrite; i++)
			{
				st.Write(bytes, 0, fileBlock);
				st.Flush();
			}
			st.Write(lastBlockBytes, 0, lastBlock);
			st.Close();
			st.Dispose();
		}

		private void US_Air_Force_5020(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			for (int j = 0; j < 3; j++)
			{
				switch (j)
				{
					case 0:
						Simple(fileInfo, callbackProgress);
						break;
					case 1:
						British_HMG_IS5_Base_Line(fileInfo, callbackProgress);
						break;
					case 2:
						ShredFileByByte(fileInfo, callbackProgress, 0xaa);
						break;
				}
			}
		}

		private void US_Army_AR380_19(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			for (int j = 0; j < 3; j++)
			{
				switch (j)
				{
					case 0:
						ShredFileByRandomByte(fileInfo, callbackProgress);
						break;
					case 1:
						ShredFileByByte(fileInfo, callbackProgress, 0x4b);
						break;
					case 2:
						ShredFileByByte(fileInfo, callbackProgress, 0xb4);
						break;
				}
			}
		}

		private void US_DOD_5220_22_M(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			for (int j = 0; j < 3; j++)
			{
				switch (j)
				{
					case 0:
						Simple(fileInfo, callbackProgress);
						break;
					case 1:
						British_HMG_IS5_Base_Line(fileInfo, callbackProgress);
						break;
					case 2:
						ShredFileByByte(fileInfo, callbackProgress, 0x97);
						break;
				}
			}
		}

		private void British_HMG_IS5_Enhanced(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			for (int j = 0; j < 3; j++)
			{
				switch (j)
				{
					case 0:
						Simple(fileInfo, callbackProgress);
						break;
					case 1:
						British_HMG_IS5_Base_Line(fileInfo, callbackProgress);
						break;
					case 2:
						ShredFileByRandomByte(fileInfo, callbackProgress);
						break;
				}
			}
		}

		//private  void FlushBuffer(string fileName, byte[] bytes)
		//{
		//    FileStream fs = new FileStream(fileName, FileMode.Open, FileAccess.Write, FileShare.None,bytes.Length);
		//    fs.Seek(0, SeekOrigin.Begin);
		//    fs.Write(bytes,0,bytes.Length);
		//    fs.Flush();
		//    fs.Close();
		//    fs.Dispose();
		//}

		private static byte RandomByte()
		{
			byte result = Convert.ToByte(RandomNumber.Next(255));
			return result;
		}

		//private  byte[] RandomBuffer(FileInfo fileInfo, int Total, int Current, ref byte A)
		//{
		//    long fileLength = fileInfo.Length;
		//    byte[] bytes = new byte[fileLength];
		//    for (long i = 0; i < fileLength; i++)
		//    {
		//        bytes[i] = RandomByte(0, 255);
		//        if (i == (int)((fileLength / ((100 / Total) * Current)) * A))
		//        {
		//            A += Convert.ToByte(1);
		//        }
		//    }
		//    return bytes;
		//}

		private void German_VSITR(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			for (int j = 0; j < 7; j++)
			{
				switch (j)
				{
					case 0:
					case 2:
					case 4:
						Simple(fileInfo, callbackProgress);
						break;
					case 1:
					case 3:
					case 5:
						British_HMG_IS5_Base_Line(fileInfo, callbackProgress);
						break;
					case 6:
						ShredFileByByte(fileInfo, callbackProgress, 0xaa);
						break;
				}
			}
		}

		private void Canadian_RCMP_TSSIT_OPS_II(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			for (int j = 0; j < 7; j++)
			{
				switch (j)
				{
					case 0:
					case 2:
					case 4:
						Simple(fileInfo, callbackProgress);
						break;
					case 1:
					case 3:
					case 5:
						British_HMG_IS5_Base_Line(fileInfo, callbackProgress);
						break;
					case 6:
						ShredFileByRandomByte(fileInfo, callbackProgress);
						break;
				}
			}
		}

		private void USDOD_5220_22_M_E_C_E(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			for (int j = 0; j < 7; j++)
			{
				switch (j)
				{
					case 0:
						ShredFileByByte(fileInfo, callbackProgress, 0xd3);
						break;
					case 1:
						ShredFileByByte(fileInfo, callbackProgress, 0x2c);
						break;
					case 2:
					case 3:
					case 6:
						ShredFileByRandomByte(fileInfo, callbackProgress);
						break;
					case 4:
						ShredFileByByte(fileInfo, callbackProgress, 0x95);
						break;
					case 5:
						ShredFileByByte(fileInfo, callbackProgress, 0x6a);
						break;
				}
			}
		}

		private void Bruce_Schneier(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			for (int j = 0; j < 7; j++)
			{
				switch (j)
				{
					case 0:
						Simple(fileInfo, callbackProgress);
						break;
					case 1:
						British_HMG_IS5_Base_Line(fileInfo, callbackProgress);
						break;
					case 2:
					case 3:
					case 4:
					case 5:
					case 6:
						ShredFileByRandomByte(fileInfo, callbackProgress);
						break;
				}
			}
		}

		private void Peter_Gutmann_Lite(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			for (int j = 0; j < 10; j++)
			{
				switch (j)
				{
					case 0:
						ShredFileByRandomByte(fileInfo, callbackProgress);
						break;
					case 1:
						ShredFileByByte(fileInfo, callbackProgress, 0x55);
						break;
					case 2:
						ShredFileByByte(fileInfo, callbackProgress, 0xaa);
						break;
					case 3:
						ShredFileByGutmannByte(fileInfo, callbackProgress, 0x92, 0x49, 0x24);
						break;
					case 4:
						ShredFileByGutmannByte(fileInfo, callbackProgress, 0x49, 0x24, 0x92);
						break;
					case 5:
						ShredFileByGutmannByte(fileInfo, callbackProgress, 0x24, 0x92, 0x49);
						break;
					case 6:
						ShredFileByByte(fileInfo, callbackProgress, 0x4b);
						break;
					case 7:
						ShredFileByByte(fileInfo, callbackProgress, 0xb4);
						break;
					case 8:
						Simple(fileInfo, callbackProgress);
						break;
					case 9:
						British_HMG_IS5_Base_Line(fileInfo, callbackProgress);
						break;
				}
			}
		}

		private void Peter_Gutmann(FileInfo fileInfo, CallbackProgress callbackProgress)
		{
			byte ByteSelect = 0x00;
			for (int j = 0; j < 35; j++)
			{
				switch (j)
				{
					case 0:
					case 1:
					case 2:
					case 3:
						ShredFileByRandomByte(fileInfo, callbackProgress);
						break;
					case 4:
						ShredFileByByte(fileInfo, callbackProgress, 0x55);
						break;
					case 5:
						ShredFileByByte(fileInfo, callbackProgress, 0xaa);
						break;
					case 6:
						ShredFileByGutmannByte(fileInfo, callbackProgress, 0x92, 0x49, 0x24);
						break;
					case 7:
					case 25:
						ShredFileByGutmannByte(fileInfo, callbackProgress, 0x49, 0x24, 0x92);
						break;
					case 8:
					case 26:
						ShredFileByGutmannByte(fileInfo, callbackProgress, 0x24, 0x92, 0x49);
						break;
					case 9:
					case 27:
						ShredFileByByte(fileInfo, callbackProgress, ByteSelect);
						break;
					case 10:
					case 11:
					case 12:
					case 13:
					case 14:
					case 15:
					case 16:
					case 17:
					case 18:
					case 19:
					case 20:
					case 21:
					case 22:
					case 23:
						ByteSelect += 0x11;
						ShredFileByByte(fileInfo, callbackProgress, ByteSelect);
						break;
					case 24:
						ByteSelect += 0x11;
						ShredFileByByte(fileInfo, callbackProgress, ByteSelect);
						ByteSelect = 0x00;
						break;
					case 28:
						ShredFileByGutmannByte(fileInfo, callbackProgress, 0x6d, 0xb6, 0xdb);
						break;
					case 29:
						ShredFileByGutmannByte(fileInfo, callbackProgress, 0xb6, 0xdb, 0x6d);
						break;
					case 30:
						ShredFileByGutmannByte(fileInfo, callbackProgress, 0xdb, 0x6d, 0xb6);
						break;
					case 31:
					case 32:
					case 33:
					case 34:
						ShredFileByRandomByte(fileInfo, callbackProgress);
						break;
				}
			}
		}

		public void Dispose()
		{
			catchBytes = null;
		}

		///<summary>Routine to shred an empty directory
		///</summary>
		///<param name="DirFullPath">Directory in full path</param>
		///<param name="recursive">true to shred subdirectories.</param>
		///<param name="callbackProgress">Callback progress; use null if not needed</param>
		///<exception cref="ArgumentNullException"></exception>
		public void ShredEmptyDirectory(string DirFullPath, bool recursive, CallbackProgress callbackProgress)
		{
			if (DirFullPath == null)
			{
				throw new ArgumentNullException("DirFullPath");
			}
			DirectoryInfo dirInfo = new DirectoryInfo(DirFullPath);
			ShredEmptyDirectory(dirInfo, recursive, callbackProgress);
		}

		///<summary>Routine to shred an empty directory
		///</summary>
		///<param name="directoryInfo">DirectoryInfo of the directory</param>
		///<param name="recursive">true to shred subdirectories.</param>
		///<param name="callbackProgress">Callback progress; use null if not needed</param>
		///<exception cref="ArgumentNullException"></exception>
		public void ShredEmptyDirectory(DirectoryInfo directoryInfo, bool recursive, CallbackProgress callbackProgress)
		{
			if (directoryInfo == null)
			{
				throw new ArgumentNullException("directoryInfo");
			}
			if (recursive)
			{
				ShredEmptyRecurssiveDirectory(directoryInfo, callbackProgress);
			}
			else
			{
				ShredEmptyDirectory(directoryInfo);
			}
		}

		internal static void ShredEmptyRecurssiveDirectory(DirectoryInfo directory2Shred, CallbackProgress callbackProgress)
		{
			if (directory2Shred.Exists)
			{
				if (directory2Shred.GetDirectories().Length == 0)
				{
					ShredEmptyDirectory(directory2Shred);
				}

				else
				{
					DirectoryInfo[] dirs;
					dirs = directory2Shred.GetDirectories();
					int iMaxDir = dirs.Length - 1;

					for (int i = iMaxDir; i >= 0; i--)
					{
						DirectoryInfo dirCurrent = dirs[i];
						ShredEmptyRecurssiveDirectory(dirCurrent, callbackProgress);
					}

					ShredEmptyDirectory(directory2Shred);
				}
			}
		}

		internal static void ShredEmptyDirectory(string DirFullPath)
		{
			if (DirFullPath == null)
			{
				throw new ArgumentNullException("DirFullPath");
			}
			DirectoryInfo dirInfo = new DirectoryInfo(DirFullPath);
			ShredEmptyDirectory(dirInfo);
		}

		internal static void ShredEmptyDirectory(DirectoryInfo directoryInfo)
		{
			if (directoryInfo == null)
			{
				throw new ArgumentNullException("directoryInfo");
			}
			if (!directoryInfo.Exists)
			{
				throw new DirectoryNotFoundException(string.Format("Directory {0} doesn't exist.", directoryInfo.FullName));
			}
			if ((directoryInfo.GetDirectories().Length > 0) || (directoryInfo.GetFiles().Length > 0))
			{
				throw new IOException(string.Format("Directory {0} is not empty.", directoryInfo.FullName));
			}

			string rndDirPath = directoryInfo.Parent.FullName + Path.DirectorySeparatorChar + Guid.NewGuid();
			if (!Directory.Exists(rndDirPath))
			{
				directoryInfo.MoveTo(rndDirPath);
			}
			else
			{
				directoryInfo.Delete(true);
			}
			DirectoryInfo info = new DirectoryInfo(rndDirPath);
			if (!info.Exists)
				return;
			try
			{
				info.Delete(false);
				return;
			}
			catch (IOException)
			{
				int num = 0;
				while (info.Exists)
				{
					if (num++ > 20)
					{
						return;
					}
					Thread.Sleep(250);
					info.Delete(false);
				}
			}
			return;
		}

		///<summary>Routine to shred files in a directory
		///</summary>
		///<param name="directoryInfo">DirectoryInfo of the directory</param>
		///<param name="shredMethod">Shredder Method</param>
		///<param name="filePattern">File pattern</param>
		///<param name="recursive">true for shred subdirectories</param>
		///<param name="callbackProgress">Callback progress; use null if not needed</param>
		///<exception cref="ArgumentNullException"></exception>
		///<exception cref="DirectoryNotFoundException"></exception>
		public void ShredFilesInDirectory(DirectoryInfo directoryInfo, ShredMethod shredMethod, string[] filePattern,
		                                  bool recursive, CallbackProgress callbackProgress)
		{
			if (directoryInfo == null)
			{
				throw new ArgumentNullException("directoryInfo");
			}
			if (!directoryInfo.Exists)
			{
				throw new DirectoryNotFoundException(string.Format("Directory {0} doesn't exist.", directoryInfo.FullName));
			}
			if (recursive)
			{
				ShredFilesRecurssiveDir(directoryInfo, shredMethod, filePattern, callbackProgress);
			}
			else
			{
				ShredFilesInDirectory(directoryInfo, shredMethod, filePattern, callbackProgress);
			}
		}

		internal void ShredFilesInDirectory(DirectoryInfo directoryInfo, ShredMethod shredMethod, string[] filePattern,
		                                    CallbackProgress callbackProgress)
		{
			foreach (string pattern in filePattern)
			{
				FileInfo[] files = directoryInfo.GetFiles(pattern);
				foreach (FileInfo filename in files)
				{
					ShredFile(filename, shredMethod, null);
				}
			}
		}

		internal void ShredFilesRecurssiveDir(DirectoryInfo directoryInfo, ShredMethod shredMethod, string[] filePattern,
		                                      CallbackProgress callbackProgress)
		{
			if (directoryInfo.Exists)
			{
				DirectoryInfo[] dirs;
				//DirectoryInfo dirCurrent;
				dirs = directoryInfo.GetDirectories();

				foreach (DirectoryInfo dirCurrent in dirs)
				{
					ShredFilesRecurssiveDir(dirCurrent, shredMethod, filePattern, callbackProgress);
				}
				ShredFilesInDirectory(directoryInfo, shredMethod, filePattern, callbackProgress);
			}
		}

		///<summary>Shred all files in a directory and remove the directory at the end.
		///</summary>
		///<param name="directoryInfo">DirectoryInfo of the directory</param>
		///<param name="shredMethod">Shredder Method</param>
		///<param name="callbackProgress">Callback progress; use null if not needed</param>
		///<exception cref="ArgumentNullException"></exception>
		///<exception cref="DirectoryNotFoundException"></exception>
		public void ShredDirectory(DirectoryInfo directoryInfo, ShredMethod shredMethod,
		                           CallbackProgress callbackProgress)
		{
			if (directoryInfo == null)
			{
				throw new ArgumentNullException("directoryInfo");
			}
			if (!directoryInfo.Exists)
			{
				throw new DirectoryNotFoundException(string.Format("Directory {0} doesn't exist.", directoryInfo.FullName));
			}
			string[] patterns = {"*"};
			ShredFilesRecurssiveDir(directoryInfo, shredMethod, patterns, callbackProgress);
			ShredEmptyRecurssiveDirectory(directoryInfo, callbackProgress);
		}
	}
}